@@ -4,7 +4,7 @@ package Database::Async::Engine::PostgreSQL;
 use strict;
 use warnings;
 
-our $VERSION = '0.006';
+our $VERSION = '0.007';
 
 use parent qw(Database::Async::Engine);
 
@@ -135,8 +135,6 @@ async sub connect {
     $self->add_child(
         my $stream = IO::Async::Stream->new(
             handle   => $sock,
-            read_len => $self->read_len,
-            write_len => $self->write_len,
             on_read  => sub { 0 }
         )
     );
@@ -151,9 +149,15 @@ async sub connect {
     $self->outgoing->each(sub {
         $log->tracef('Write bytes [%v02x]', $_);
         $self->ready_for_query->set_string('');
-        $self->stream->write($_);
+        $self->stream->write("$_");
+        return;
     });
-    $stream->configure(on_read => $self->curry::weak::on_read);
+    $stream->configure(
+        on_read   => $self->curry::weak::on_read,
+        read_len  => $self->read_len,
+        write_len => $self->write_len,
+        autoflush => 0,
+    );
 
     $log->tracef('Send initial request with user %s', $uri->user);
     my %qp = $uri->query_params;
@@ -206,10 +210,8 @@ async sub negotiate_ssl {
     if($resp eq 'S') {
         # S for SSL...
         $log->tracef('This is SSL, let us upgrade');
-        my $sock = $stream->write_handle;
-        $stream->configure(handle => undef);
-        my $ssl_sock = await $self->loop->SSL_upgrade(
-            handle          => $sock,
+        $stream = await $self->loop->SSL_upgrade(
+            handle          => $stream,
             # SSL defaults...
             SSL_server      => 0,
             SSL_hostname    => $self->uri->host,
@@ -218,7 +220,6 @@ async sub negotiate_ssl {
             # better than we do
             (map {; $_ => $self->{$_} } grep { /^SSL_/ } keys %$self)
         );
-        $stream->configure(handle => $ssl_sock);
         $log->tracef('Upgrade complete');
     } elsif($resp eq 'N') {
         # N for "no SSL"...
